***********************************************************************
*                         LAYER5.5.f                                   *
*     version March 2007:                                              *
*     finite element solution of a 2D viscous flow by the penalty      *
*         function method, compression and basal shear                 *
*         Laterally varying temperature
*         Flag for temperature profile
*     second order time integration                                    *
*     strength envelope: Byerlee laws or constant strength             *
*                        power law creep                               *
*                        velocity weakening                            *
*     Shear gradient boundary conditions                               *
*     Adapted for icy satellites
*     Laurent Montesi, after visct.f by Maria Zuber and Greg Neumann   *
*     only for 4-nodes elements , vertical ordering                    *
*                                                                      *
*     5.3 : Strain weakening attempt with initial strain               *
*           Overburden now accounts for variable density in layers     *
*           Can plot strain in ps output.                              *
*           (Matthias Delescluse, december 2005)                       *
*     5.5 : Weakening subroutine for selective abandonment             *
*	    See Delescluse et al. (2008)                                 *
************************************************************************
*     VARIABLES                                                        *
*     amp: amplitude of the perturbation at moho(from top)=layer imoho *
*                or y=ymoho                                            *
*     bf      body force at element nodes                              *
*             bf(i,1)=fx(i);   bf(i,2)=fy(i),   i=1, npe               *
*     c1      effective viscosity                                      *
*     c2      penalty function                                         *
*     f0, cfmax   Byerlee slope at reference, maximum allowed          *
*     cf2     log10(v/v0) velocity weakening constant                  *
*     cmin/cmax   max and min allowed viscosity  (log10 in .in file    *
*     cmaxwell   ???? (log10 in .in file)                              *
*     cmu(i,j)initial (?) power law effective viscosity                *
*     cm(i,j) power law effective viscosity                            *
*     dGdx    lateral gradient in surface geotherm Gs                  *  
*     dtmx    lateral gradient in mantle temperature                   *
*     dx,dy   initial spacing, maybe never used                        *
*     e2inv, e2(i)  reference strain invariant, and value for element i*
*     elxy    global coordinates of element nodes:                     *
*             elxy(i,1)=xe(i); elxy(i,2)=ye(i), i=1, npe               *
*     Fs      Surface heat flux                                        *
*     flag(i) 1 if element i fails, 0 if not                           * 
*     free    1 if "slippery"walls (no es), 0 otherwise                *
*     fT      flag for temperature                                     *
*             = 1 linear Ts, Gs, limited to tm                         *
*             = 2 error function, Ts, Gs, asymptotic to tm             *
*             = 3 exponential (for ice)  Ts, Gs, limited to tm         *
*     grav, rhog    body force intensity, for body force and           *
*                               Coulomb criterion                      *
*     gf(i)   assembled 'force' vector.  also used to return the       *
*             solution from subroutine 'solve'                         *
*     Gs      surface temperature gradient (K/km)                      *
*     gstif   global coefficient (or 'stiffness') matrix.  note        *
*             that it is assembled and stored in a banded form         *
*     ibdf(i) array of specified boundary degrees of freedom           *
*     iel     1 for linear quadrlateral elements                       *
*             2 for quadratic quadrilateral elements                   *
*     ibrit   1 for Byerlee's law                                      *
*             2 for constant strength                                  *
*     iprob   0 show stress                                            *
*             otherwise show viscosity                                 *
*     imesh   = 0  mesh information is to be read                      *
*             = 1  mesh is generated by the program                    *
*             = 2  " " and viscosities are read as log10 of value      *
*     integr  = 0  calculate perturbed x,y at half time step           *
*             = 1  calculate new solution at full time step            *
*     kT      = 0.567: T-dependent conductivity is kT/temp               *
*     mplot    modulo of plots generated, ie when mod(iplot,mplot)=0   *
*     nhbw    half band width of the coefficient matrix, gstif         *
*     ncmax   max column dimension of gstif                            *
*     ndf     number of degrees of freedom per node:                   *
*                  ndf=2, for elasticity                               *
*     nem     # of elements in the mesh                                *
*     neq     total number of equations in the problem (=nnm*ndf)      *
*     nlayer  # of line for each layer                                 *
*     nmat    # of different materials (# layers)                      *
*     nn      total degrees of freedom per element (=npe*ndf)          *
*     nnm     # of nodes in the mesh                                   *
*     nod(i,j)   global node # corresponding to jth node of ith element*
*     nod1,n25   first plotted element, right plot limit?              *
*     npe     # of node/element                                        *
*     nplot   number of full iterations, iplot=iteration count         *
*     nprnt   >=3 if want to save initial shear and norma stress       *
*             >=2 if want to save initial temperature                  *
*             >=1 if want to save initial mesh parameters              *
*             <-1 if want to plot velocity                             *
*     nrbl,nrb(i)     number of brittle failure laws                   *
*     nrdl,nrd(i)     number of ductile failure laws                   *
*     nrmax   max row dimension of gstif                               *
*     nsdf    number of specified primary degrees of freedom           *
*     nx,ny   # of elements in the x and y direction                   *
*     nx1,ny1 # of elements at edges in the x and y direction          *
*     pshort  percentage of strain rate due to shortening              *
*     SC(i,j),SF(i,j): Byerlee/strength parameter/element              *
*     scale   distance in the x direction (km)                         *
*     start   initial (negative) time for model convergence            *
*     ratio   dy/dx for the uppermost elements                         *
*     relax   accelerator                                              *
*     rho(i)  density of element, normalized to mantle                 *
*     sn(i)   power law exponent                                       *
*     S0      cohesion                                                 *
*     temp(i)  temperature of the element i (K)                        *
*     ts, tm   surface and infinity temperature  (K)                   *
*     U0       end velocity                                            *
*     UT       top velocity                                            *
*     vbdf(i)  array of the values corresponding to the specified      *
*                       degrees of freedom in array ibdf(i)            *
*     x(i), y(i)    position of element i                              *
*     xloc(i),yloc(i): position of node i for a certain element (plot) *
*     w1,w2,w3,w4   window limit (km)                                  * 
*     w(i,j)  value of the i-th degree of freedom at the j-th node     *
*                     of the element                                   *
*                                                                      *
************************************************************************
*     dimensions                                                       *
*        the program won't do anything bad if you keep nhbw<ncmax,     *
*     nx<nxmax, ny<nymax, nfo<nsdf                                     *
*        this version is intended for nxmax=100, nymax=80, leading to  *
*     ncmax=224 and ndf = 360 (but watch out!)                         *
*                                                                      *
************************************************************************

* ---------------------   Variables declaration   -------------------- *

      implicit real *8 (a-h,o-z)
* used constants
      parameter(pi=3.14159265358979,r=8.314,yr=86400.*365.24)
*      parameter (strnrcp=1.d13,strnrate=1.d-13)
      parameter (strndef=1.d-13)
c      parameter (rt2=1.414213562373*strnrate)
      parameter (alpha=3.d-05)
      parameter (zero=0.d1)
* maximum Pb dimensions
c      parameter (nxmax=400,nymax=40,nxmx1=nxmax+1,nymx1=nymax+1)
c      parameter (nxmax=300,nymax=40,nxmx1=nxmax+1,nymx1=nymax+1)
c      parameter (nxmax=200,nymax=40,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nxmax=1200,nymax=200,nxmx1=nxmax+1,nymx1=nymax+1)
      parameter (nmax=nxmax*nymax,nemmax=nxmx1*nymx1)
      parameter (nrmax=2*nemmax, ncmax=2*nymax+6, nfo=4*(nxmax+nymax+1))
c      parameter (S0o=0.,S1o=50.,cf0o=0.85,cf1o=0.6)
      parameter (npe=4,iel=1)
      character*27 readfile,mess,codename,strainfile
      character*32 meshfile,tempfile,outfile,psfile,effvisc,fname,bcfile
c,obsfile
      
      real*8  e2(nmax),str2(nmax),str2t(nmax),str2i(nmax)
      real*8 S0(5),f0(5),weako(5),wmaxo(5),L
      real*8 xp(nemmax),yp(nemmax),rhowotemp(nmax)
      real*8 u(2,nemmax), maxobd
      real*4 rho(nmax),temp(nmax)
      real*4 cmu(nmax,5),cmum(nmax),sn(nmax,5),cm(nmax,5),cmin,cmax
      real*4 q(5),expon(5),B(5),wside
      integer nrd(nmax),nrb(nmax)
      real*4 SC(nmax,5),SF(nmax,5),weak(nmax,5),wmax(nmax,5)
      real*4 iwstrain(5),iwstraint(nmax,5),weakS(5),weakSt(nmax,5)
      real w1,w2,w3,w4
      real*4 xloc(9),yloc(9),thick(nxmax),epsil0(5),epsil0n(nmax,5)
      real*4 coeff,epsilv,epsill,ws,wsr,sinv,xpass,xmaxi,vmcf
      integer fT,heal,healc,wmode,itec,iwstr
      integer flag(nmax),flg(nmax),start,comp,free

      dimension ibdf(nfo),vbdf(nfo)
      dimension Bmu(nymax),CD(nymax)
c,v(2,9),ibsf(nfo),vbsf(nfo)

      common /meshval/x(nemmax),y(nemmax)
      common /nodva/nod(nmax,9)
      common /forces/ndf,gstif(nrmax,ncmax),gf(nrmax)
      common /lstif/elstif(18,18),elxy(9,2),bf(9,2),f(18),w(2,9),rho
      common /io/ire,iwr
      common /coverbd/celload(nymax)

*                       __________________                             *
*                      /                  \                            *
* -----------------------  PREPROCESSOR  ----------------------------- *
*                      \__________________/                            *
      rt3=sqrt(3.d0)

      ire=1
      ist=7
      iwr=9
* initialize the matrixes

* All there is to worry about what files we will handle...-------------
      if(iargc().ne.0) then
         call getarg(1,readfile)
      else
         print *,'filename.in?'
         read (*,'(a)') readfile
      endif

      i=index(readfile,'.')-1
      tempfile = readfile(1:i)//'.temp'
      meshfile = readfile(1:i)//'.mesh'
      bcfile = readfile(1:i)//'.bc'
      fname=readfile(1:i)
      print *,fname

      effvisc = readfile(1:i)//'.eff'
      psfile  = readfile(1:i)//'.ps'
      outfile = readfile(1:i)//'.out'
      strainfile= readfile(1:i)//'.str'

      open (unit=iwr, file=outfile)
      open(unit=ire, file=readfile)
      open(unit=ist, file=strainfile)
      write(iwr,'(a,a,a,a,a)') readfile,tempfile,meshfile,psfile,bcfile

      read (ire,'(a)') codename
      if (codename.ne.'LAYER-5.5') then
         print *,'input file not intended for ''LAYER5.5'''
         stop
      endif

* take care about the mesh, bordel! ------------------------------------

* some mesh size precautions...
      read (ire,*) nx,ny
      read (ire,*) nyl,nxl,iloc
c      if ((nx.gt.100).or.(ny.gt.80).or.(npe.gt.9)) then
      if ((nx.gt.nxmax).or.(ny.gt.nymax).or.(npe.gt.9)) then
         print *,'This program is not intended for more than:'
         print *,'nx=',nxmax,', ny=',nymax,', npe=9, less than your
     &        input file...'
         stop
      endif

* some parameters that we'll need
      nx1=nx+1
      ny1=ny+1
      nem=nx*ny
      nem1=nx1*ny1

* more precautions
      if (iloc.eq.ny1) then
         print *,'I can''t handle a perturbation at the bottom.'
         stop
      endif

      nxx1 = iel*nx + 1
      nyy1 = iel*ny + 1

* build the MESH and the NOD matrix
      call xygen(nem1,nx1,ny1,nxl,nyl,ire,iloc)
      call nodes(nx,ny,iel,npe,nnm)

* TEMPERATURE initialisation
      read (ire,*) ts,tm,Gs,dGdx,dtmx,fT
      print *, ' ts,tm,Gx,dGdx,dtmx, fT=', ts,tm,Gs,dGdx,dtmx,fT

* loop over the elements, temperature calculation
      do i=1,nem
         str2(i)=0.d0
         str2t(i)=0.d0
         call centroid (i,cen,cenx,0.,1,nx,ny,nem,npe)
         if (fT.eq.2) then
*     error function
            del =2.*(tm-(dtmx*cenx)-ts)/((Gs-dGdx*cenx)*sqrt(pi))
            z=cen/del
            temp(i)=ts+(tm-(dtmx*cenx)-ts)*erf(z)
         elseif (fT.eq.3) then
*     exponential profile
            temp(i)=min(ts*exp(cen*(Gs-dGdx*cenx)/ts),(tm-dtmx*cenx))
         else
*     linear geotherm
            temp(i)=min(ts+cen*(Gs-dGdx*cenx),(tm-dtmx*cenx))
         endif
      enddo
 
* calculate dimension, half band-width
      ndf=2
      neq=nnm*ndf
      nn=npe*ndf
      nhbw = 0
      do 100 n=1,nem
         do 100 i=1,npe
            do 100 j=1,npe
               nw = (iabs(nod(n,i)-nod(n,j))+1)*ndf
               if(nhbw.lt.nw) nhbw=nw
 100  continue
      print *,'ncmax,nhbw,neq,nnm= ',ncmax,nhbw,neq,nnm

      nloc=ny1-iloc
      write (iwr,*) 'mesh:',nx,' X',ny,', scale',x(nem1),'*', y(1),'
     &km'
      write (iwr,*) 'perturb: amp=',amp,' @ y (',iloc,' )=',y(nloc)

*     convert mesh to meter
      do i=1,nem1
         x(i)=x(i)*1000
         y(i)=y(i)*1000
      enddo

* save the temperatures in tempfile ------------------------------------
      open (unit=2,file=tempfile)
      i1=1
      i2=ny
      do i=1,nx
         write (2,'(10f8.2)') (temp(j), j=i1,i2)
         i1=i1+ny
         i2=i2+ny
      enddo
      close (unit=2)

* reference state
      read(ire,*) c1, c2, relax0, cmin, cmax, cmaxwell
      relax=relax0
      c2 = c2*1.0d7
      print *,'cmaxwell:',cmaxwell
      read(ire,*) strnrate,ec,es,delt,freeh,freev
      print *,strnrate,ec,es,delt,freeh,freev
      if ((es.ne.0).and.(freev.eq.1)) then
         print *,'Careful, free slip vertical walls and
     &    shear are not compatible, switch for freev=0.'
        freev=0
      endif
* conversions
      strnrate=strnrate/yr
      strnrcp=1/strnrate
      cmin=10.**cmin
      cmax=10.**cmax
      cmaxwell=10.**cmaxwell
      delt=delt*yr
      H=(y(1)-y(nyy1))
      L=(x((nxx1-1)*nyy1+1)-x(1))
      U0=ec*strnrate*L
      US=es*strnrate*H
      e2inv=sqrt(es**2+ec**2)
* element rheology -----------------------------------------------------
      read (ire,*) grav,rhog

* determine rheology structure law ---
      read(ire,*) nmat 
      oldj=0
      do layer=1,nmat
* 5.3: rho normalized to mantle is now read here
         read(ire,*) nlayer, nrbl, nrdl, rhop
         print *,'brittle and ductil rheologies  (from bott): '
     &        ,nrbl,nrdl
         if(layer.eq.1) nlay1 = nlayer+1
* determine brittle strength  ---
         if (ec.le.0) then
            comp=1
         else
            comp=0
         endif


         print *,'Cohesion, Friction, weakening coeff and max'
         do irb=1,nrbl
            read (ire,*) S0o,f0o, weako(irb), wmaxo(irb)
	    read(ire,*) iwstrain(irb),weakS(irb),epsil0(irb)

	print *, iwstrain(irb),weakS(irb)

            talpha0=tan(0.5*((0.5*pi)+atan(f0o)))
*     scale brittle law in compression or extension.
            if (comp.eq.1) then
               S0(irb)=S0o*talpha0
               f0(irb)=(talpha0**2-1)/2
            else
               S0(irb)=-S0o/talpha0
               f0(irb)=(talpha0**2-1)/(2*talpha0**2)
            endif

            print *,S0(irb),f0(irb),weako(irb),wmaxo(irb)
            print *,iwstrain(irb),weakS(irb),epsil0(irb)
            		

         enddo
* ductile flow law         
         print *,'n,A,B,q: '      
         do ird=1,nrdl
*     read material power-law exponents, pre-x, activation energy, rhop
*     is now read before 
            read(ire,*) expon(ird), a, q(ird)
*     convert to Pa**(n)/s as in Chen and Morgan (1990)
            A=a*10.**(-6.*expon(ird))
*     now, I do it my way
            B(ird)=(rt3/6)*(2/(A*rt3))**(1./expon(ird))
            print *, expon(ird),a,b(ird),q(ird),rhop
         enddo
         do j=1,nlayer
            ind=j+oldj
*     store characteristics
            do i=1,nx
               rho(ind)=rhop/(1.+alpha*(temp(ind)-273.65))
* 5.3: store cell densities
               rhowotemp(ind)=rhop
               nrb(ind)=nrbl
               nrd(ind)=nrdl
               do ird=1,nrdl
                  sn(ind,ird)=expon(ird)
                  arg = min(q(ird)/(expon(ird)*r*temp(ind)), 75.d0)
                  cmun=strnrate**(-1.+1./expon(ird))*B(ird)*exp(arg)
                  cmun=min(cmun, cmax)
                  cmu(ind,ird)=max(cmun, cmin)
               enddo
               do irb=1,nrbl
                  SC(ind,irb)=S0(irb)
                  SF(ind,irb)=f0(irb)

		  iwstraint(ind,irb)=iwstrain(irb)		  
		  weakSt(ind,irb)=weakS(irb)
		  epsil0n(ind,irb)=epsil0(irb)
                  weak(ind,irb)=weako(irb)
                  wmax(ind,irb)=wmaxo(irb)
               enddo
               ind=ind+ny
            enddo
         enddo
         oldj=oldj+nlayer        
      enddo
      print *,'nem,ind: ',nem, ind
      do i=1,nem
         cmum(i)=cmax
         do j=1,nrd(i)
            cm(i,j)=cmu(i,j)*c1
            cmu(i,j)=cm(i,j)
            cmum(i)=min(cmum(i),cmu(i,j))
         enddo
      enddo
* weak margin

* Show some critical numbers
      DT=tau0/(rho(ind-ny)*rhog*H)
c      print '(5e12.4)',DT,tau0,rho(ind-ny),rhog,H
      print *,'Reference strain rate and its inverse: ',
     &     strnrate,strrcp,'/s'
      print *,'horizontal strain rate ec/ref: ',ec
      print *,'mesh dimensions: ',L/1000,'km x ',H/1000,'km'
      print *,'Bottom velocity: ',U0*yr*1000,'km/Ma'
      print *,'Top velocity: ',US*yr*1000,'km/Ma'
      print *,'Dynamic topography: ',DT*L/1000,'km'


      read(ire,*) straininit,vmcf

* calculation parameter
      read (ire,*) start,nplot,mplot, nod1, n25
      print *,start,nplot,mplot, nod1, n25
      iplot=start
      iplotpage=0
      delstrain= ec*delt*strnrate

* initialize plot file
      read (ire,*) w1,w2,w3,w4
      open(unit=8,file=psfile)
      call window(w1,w2,w3,w4)
 
* the options left behind
      read (ire,*) iprob, imesh, nprnt, negplot

* 5.3:read pre-existent weakness spacing and initialize strain
* also read water layer depth and max depth of weakness
      read (ire,*) spacestrain, depthw,depthweak, wmode, wside
      spacestrain=spacestrain*1000.
      depthweak=depthweak*1000.
      wside=wside*1000.

      print *, 'water depth: ',depthw,' km, wside=',wside

* 5.3: max overburden:
      maxobd=0
      do i=1,ny
         celload(i)=celload(i)*rhowotemp(i)*rhog
         maxobd=maxobd+celload(i)
      enddo

* 5.3: water part (a detail)
      maxobd=maxobd+ depthw*1000.*9.8*1000.


* 5.3: first introduction of strain: initial distribution of
*      weakness
      jspace=0
      jcnt=0

* 5.3: quantity to add for symmetry
      vtest=1./2.*(x(nem1)-
     $           int(x(nem1)/spacestrain)*spacestrain)
      print *, vtest
      
         if (wmode.gt.0) then
      do i=1,nem1-(ny+1)
	if (x(i).lt.(x(nem1)-wside)) then
           if (y(i).lt.depthweak) then
               
              if ((dble(jspace)*spacestrain+vtest+wside).lt.(x(i))) then
c                 print *,i,y(i)
                 if (y(i).le.0.)then
                    jcnt=jcnt+1
		    itec=0
                    if (jcnt.eq.3) then
                       jspace=jspace+1
                       jcnt=0
                       print *, x(i)
                    endif
                 else
* 5.3: initial strain repartition 
       if ((jspace.ne.0).and.(jspace.ne.int(x(nem1)/spacestrain))) then
                if (wmode.eq.1) then
		  str2(i-i/(ny+1))=straininit/(strnrate*delt)
                  str2t(i-i/(ny+1))=straininit
		endif
		if (wmode.eq.2) then
		  str2(i-i/(ny+1)-itec*ny)=straininit/(strnrate*delt)
                  str2t(i-i/(ny+1)-itec*ny)=straininit	
		  itec=itec+1	  
		endif
       endif
* 5.3: adaptation from knot grid to cell grid here
                 endif

              endif

            endif

	endif
      enddo
      	endif
       
      close (unit=ire)



* BOUNDARY CONDITIONS --------------------------------------------------
      print *,'Horizontal wall free =',freeh
      print *,'Vertical wall free =',freev
* Initialize BC index
      ind=0
* Boundary type: bottom boundary ---------------------------------------
* If no-slip walls (required if shear present)
      if (freeh.eq.0) then 
         print *,'Bounded bottom boundary'
* fixed uniform compression at bottom boundary -------------------------
         do ix=1,nxx1
            ind=ind+1
            vbdf(ind)=0.
            ibdf(ind)=2*((ix-1)*nyy1+1)
            ind=ind+1
            vbdf(ind)=U0*(x((ix-1)*nyy1+1)-x(1))/L
            ibdf(ind)=2*((ix-1)*nyy1+1)-1
         enddo    
* If free slip walls
      else
         print *,'Free slip bottom boundary'
         do ix=1,nxx1
            ind=ind+1
            vbdf(ind)=0.
            ibdf(ind)=2*((ix-1)*nyy1+1)
         enddo 
      endif
* set the vertical boundary condition ----------------------------------
      iright=2*(nxx1-1)*nyy1
* lowermost row of nodes
         iy=0
*     left wall
         print *,iy,y(iy+1),U0
         ind=ind+1
         ibdf(ind)=2*iy+1
         vbdf(ind)=Unow
*     right wall
         ind=ind+1
         ibdf(ind)=iright+2*iy+1
         vbdf(ind)=U0
* If no-slip walls (required if shear present)
      if (freev.eq.0) then 
         print *,'Bounded walls'
*     move up think left column, ix=1
         do iy=1,ny
            do ix=1,nx
               e2((ix-1)*ny+iy)=e2inv
            enddo
*     assume iel=1
*     left wall
            Unow=-US*
     &           (y(iy+1)-y(1))*
     &           (y(iy+1)-y(1)+2*H)/
     &           (H**2)
            print *,iy,y(iy+1),Unow,U0,US
            ind=ind+1
            ibdf(ind)=2*iy+1
            vbdf(ind)=Unow
            ind=ind+1
            ibdf(ind)=2*iy+2
            vbdf(ind)=U0*(y(1)-y(iy+1))/L
*     right wall
            ind=ind+1
            ibdf(ind)=iright+2*iy+1
            vbdf(ind)=U0+Unow
            ind=ind+1
            ibdf(ind)=iright+2*iy+2
            vbdf(ind)=U0*(y(1)-y(iy+1))/L
         enddo
* if free-slip vertical wall, can''t have shear
      else
         print *,'Free slip walls'
*     move up think left column, ix=1
         do iy=1,ny
            do ix=1,nx
               e2((ix-1)*ny+iy)=e2inv
            enddo
*     assume iel=1
*     left wall
            print *,iy,y(iy+1),U0
            ind=ind+1
            ibdf(ind)=2*iy+1
            vbdf(ind)=Unow
*     right wall
            ind=ind+1
            ibdf(ind)=iright+2*iy+1
            vbdf(ind)=U0
         enddo
      endif
* free top boundary


      nsdf = ind
      print *,'degrees of freedom nsdf,ibdf: ',nsdf,ibdf(nsdf)
      print *,'iterate w/-timesteps first to converge lith properties'
      print *, '   '

* integr=0/1 for half/whole step
      integr=0

* save the mesh in meshfile
      open (unit=2,file=meshfile)
      write(2,*) nx,ny
      write(2,*) 'w=',x(nx1),' amp=',amp,'y(',iloc,')=',y(nloc),
     &' dtdz',dtdz
      i1=1
      i2=ny1
      do i=1,nx1
         write (2,'(10e10.3)') (x(j), j=i1,i2)
         i1=i1+ny1
         i2=i2+ny1
      enddo
      i1=1
      i2=ny1
      do i=1,nx1
         write (2,'(10e10.3)') (y(j), j=i1,i2)
         i1=i1+ny1
         i2=i2+ny1
      enddo
      close (unit=2)

* care about the output file, same output & order as with visct--------

       write(iwr,660) iel,npe,iprob,imesh,nprnt
       write(iwr,*)'nxx1,nyy1', nxx1, nyy1
       write(iwr,450) nem,nnm,neq
       write(iwr,560)
       write(iwr,655) c1,c2, log10(cmin), log10(cmax),U0*1000*yr
       write(iwr,*) delt/yr,grav,rhog
       if(nprnt.gt.0) 
     & write(iwr,'(i5,2f10.3,e10.3)')(i,temp(i),rho(i),cmum(i),i=1,nem)
       if(nprnt.gt.0)
     & write(iwr,460) nsdf, (i,ibdf(i),vbdf(i),i=1,nsdf)
       write(iwr,590) nhbw
*                         _______________                              *
*                        /               \                             *
* -------------------------  PROCESSOR  ------------------------------ *
*                        \_______________/                             *

*********************** FALL INTO MAIN LOOP ****************************

 101  continue
      
* 5.3: enable or disable strain healing
      if (heal.eq.1) then
         if (iplot.lt.0) then
            healc=0
         else
            healc=1
         endif
      else
         healc=0
      endif

* Au boulot, We've got a glob. stiffness. matrix to build! -------------

* initialize global stiffness matrix 
      do 130 j=1,nhbw
      do 130 i=1,neq
	gstif(i,j) = 0.0
 130  continue
      do i=1,neq
	gf(i)=0.0
      enddo

* 5.3: initialisation for overburden calculation
      icolold=1
      rhogez=maxobd

* loop over elements to calculate and assemble global stiffness matrix
      do 300 n=1,nem

* FILLS ELXY, W, AND BF FOR ELEMENT N ---------------------------------
* 5.3: new centroid returning current column
         call centroidint (n,icol,Sum,cenx,grav,3,nx,ny,nem,npe)

* EFFECTIVE VISCOSITY, RELEVANT RHEOLOGY -------------------------------
         
c* Dieterich: strain-dependent rheology, brittle strength S

         S=1.d99
         flb=0;

*  5.3:Depth summation of overburden with variable densities 
* 5.3:To reset overburden to max overburden when new column is reached:
         if (icol.gt.icolold) then
            icolold=icol
            rhogez=maxobd
         endif

         rhogez=rhogez-celload(n-(icol-1)*ny)
 
         do irb=1,nrb(n)

* 5.5: strain rate and/or strain weakening:
        coeff=CoeffStr
	iwstr=iwstraint(n,irb)
	epsilv=str2t(n)
	epsill=epsil0n(n,irb)
	ws=weakSt(n,irb)
	wsr=weak(n,irb)
	sinv=e2(n)
	xpass=x(n)
	xmaxi=x(nem1)
	call weakening(coeff,vmcf,xpass,xmaxi,iwstr,epsilv,epsill,
     & ws,wsr,sinv)

	CoeffStr=coeff
          St=(SC(n,irb)+SF(n,irb)*rhogez)*min(wmax(n,irb),CoeffStr)
            if (St.lt.S) then
               flb=irb
               S=St
            endif            
         enddo
* power law effective viscosity at strain rate invariant
         c1=cmax
         fld=0
         do ird=1,nrd(n)
            c1t=cmu(n,ird)*(1./e2(n))**(1.-1./sn(n,ird))
            if (c1t.lt.c1) then
               fld=ird
               c1=c1t
            endif
         enddo
         if(c1.lt.cmin) then
            c1=cmin
         endif

* If failure, reduces the viscosity like it should be

         taumax=2*c1*e2(n)*strnrate
         if (taumax.gt.S) then
            c1=S/(2*e2(n)*strnrate)
            flag(n)= -flb
         else
            flag(n)=fld
         endif
* over-relaxation on 1./cm ????????????????????????????????????????????
         cmn=cm(n,fld)
         diff=1./c1-1./cmn
         if(diff.gt.0.) then
            c1=cmn/(1.+relax*diff*cmn)
         else
            c1=cmn+relax*(c1-cmn)
         endif
         cm(n,fld)=c1
         c2x = c1*c2
* get the "local" stiffness matrix
         call stiff(npe,nn,iel,c1,c2x)
* ASSEMBLE GLOBAL STIFFNESS MATRIX ------------------------------------
         do 280 i=1,npe
            nr = (nod(n,i)-1)*ndf
            do 280 ii=1,ndf
               nr = nr + 1
               l  = (i-1)*ndf + ii
               gf(nr) = gf(nr) + f(l)
               do 260 j=1,npe
                  ncl = (nod(n,j)-1)*ndf
                  do 260 jj=1,ndf
                     m  = (j-1)*ndf + jj
                     nc = ncl + jj + 1 - nr
* matrix symmetric: keep only upper and diagonal terms
           if(nc.gt.0) gstif(nr,nc) = gstif(nr,nc) + elstif(l,m)
 260           continue
 280     continue

* end of the element loop: gstif and gf are done
 300  continue

* APPLY BOUNDARY CONDITIONS -------------------------------------------
      if(nsdf.gt.0) then
        call bndy(neq,nhbw,nsdf,ibdf,vbdf)
      endif
* SOLVE THE SYSTEM OF EQUATIONS ---------------------------------------
* solution returned in array gf
      call solve(neq,nhbw,0)
*                       ___________________                            *
*                      /                   \                           *
* -----------------------  POSTPROCESSOR  ---------------------------- *
*                      \___________________/                           *


 900  continue
* calculation of effective viscosity - gf has valid data only after solve...
      
* CALCULATION OF THE STRAIN RATES -------------------------------------
      biggest=0.
      smallest = 1000000000.
      biggeststr=0.
      smalleststr=1000000000.

      do n=1, nem
* need the elxy and w of the element
         call centroid (n,Sum,cenx,grav,2,nx,ny,nem,npe)
* second invariant of strain rate:
         call strainr(str2inv,e2inv,n,npe,elxy,w)

* 5.3: uses strain rate to calculate strain:
         if (iplot.ge.0.and.integr.eq.0) then
* 5.3: adding str rate*time step to strain 
         str2t(n)=str2t(n)+e2inv*delt
c         print *,e2inv*delt
* 5.3: relative strain compared to average strain
         str2(n)=str2t(n)/(strnrate*dble(iplot+1)*delt)
         endif
         e2inv=e2inv*strnrcp
         biggest=max(biggest,e2inv)
         smallest=min(smallest,e2inv)
	
	bigstr2=max(bigstr2,str2inv)
	biggeststrt=max(biggeststrt,str2t(n))
	smalleststrt=min(smalleststrt,str2t(n))
        biggeststr=max(biggeststr,str2(n))
        smalleststr=min(smalleststr,str2(n))
* save strain rate invariant for element nsolve...
         e2(n)=e2inv

* just a try: (not operational)
	 str2i(n)=str2i(n)+str2inv
	
      enddo	



* total infinitesimal horizontal strain 
      strain= (iplot*U0*delt)/sqrt((x((nxx1-1)*nyy1+1)-x(1))**2 +
     $     (y(nyy1)-y(nyy1*nxx1))**2)

* 5.3: () added for second line (**2 was only for y(nyy1*nxx1)?)
      
* total finite horizontal strain
      fstrain= iplot*delstrain  

* print that on screen
      step=iplot-0.5*integr
      print *,iplot,strnrate,delt
c      print *,biggeststr,smalleststr,biggeststrt,smalleststrt
      print 666,step,biggest,smallest,strain,fstrain




* SECOND ORDER INTEGRATION AND (MAYBE) PLOT-----------------------------
      if (integr.eq.0) then
* ----------------------------------------------------------------------
* look at surface min, max
         ymax=-1.e36
         ymin= 1.e36
         do 1040 i=nnm-nx,nnm
            ymin=min(ymin,y(i))
            ymax=max(ymax,y(i))
 1040    continue

*  save model x,y in xp,yp, velocities in u and take half step
         do 1050 i=1,nnm
            xp(i)=x(i)
            yp(i)=y(i)
            u(1,i)=gf(2*i-1)
            u(2,i)=gf(2*i)
 1050    continue

* PLOT MESH (normal and shear stresses) -------------------------------
* (when it is the right time only!) -----------
         if ((nprnt.ge.3).and.(iplot.eq.0 .or. iplot.eq.start)) then  
*     Get stress tensor and plot on extra page
*     initialize, scales

            iplotpage=iplotpage+1
            call pagelast(iplotpage)
*     titles to appear at top of each plot

            write(8,8831) fstrain,grav,Fs,fname
*     Set up scaling for plotting in kilometers
            write(8,'(a)')'gsave'
            write(8,'(a)')'x y translate'
            write(8,'(a)')'xscale yscale scale' 

*     5.3: initialisation for overburden calculation
      icolold=1
      rhogez=maxobd

*     recalc all the elxy and w, no top position and body forces this time
            do 390 n=1,nem
*     need centroid, elxy, and w
*     5.3: new version of centroid returning grid column for n
               call centroidint(n,icol,Sum,cenx,grav,2,nx,ny,nem,npe)
*     Dieterich: strain-dependent rheology, brittle strength S

               S=1.d99
               flb=0

*      5.3:Depth summation of overburden with variable densities
*      5.3:To reset overburden to max overburden when new column is reached:                
               if (icol.gt.icolold) then
                  icolold=icol
                  rhogez=maxobd
               endif

               rhogez=rhogez-celload(n-(icol-1)*ny)

               do irb=1,nrb(n)

*      5.5:determines strain rate and/or strain weakening 
        coeff=CoeffStr
	iwstr=iwstraint(n,irb)
	epsilv=str2t(n)
	epsill=epsil0n(n,irb)
	ws=weakSt(n,irb)
	wsr=weak(n,irb)
	sinv=e2(n)
	xpass=x(n)
	xmaxi=x(nem1)
	call weakening(coeff,vmcf,xpass,xmaxi,iwstr,epsilv,epsill,
     & ws,wsr,sinv)

	CoeffStr=coeff

               St=(SC(n,irb)+SF(n,irb)*rhogez)*min(wmax(n,irb),CoeffStr)
                  if (St.lt.S) then
                     flb=irb
                     S=St
                  endif
               enddo
*     second invariant of strain rate:
               call strainr(str2inv,e2inv,n,npe,elxy,w)
*     normalize to reference
               e2inv=e2inv*strnrcp
*     calculation of effective viscosity
*     - gf has valid data only after solve strain-dep. visc.
               c1=cmax
               fld=0
               do ird=1,nrd(n)
                  c1t=cmu(n,ird)*(1./e2inv)**(1.-1./sn(n,ird))
                  if (c1t.lt.c1) then
                     fld=ird
                     c1=c1t
                  endif
               enddo
*     Coulomb failure crit. again, with flag=1 for failure

               taumax=2*c1*e2(n)*strnrate

               if (taumax.gt.S) then
                  c1=S/(2*e2(n)*strnrate)
                  flag(n)=-flb
                  flg(n)=0
               else
                  flag(n)=fld
                  flg(n)=1
               endif

               if(nprnt.ge.1) then
                  c2x= c1*c2
*     stress output directly written in .out files
                  call stress(n,npe,elxy,w,c1,c2x,iel,sxx,sxy)
*     want the position of the nodes, xloc-yloc
                  do i=1,npe
                     ni = nod(n,i)
                     xloc(i)   = x(ni)
                     yloc(i)   = y(ni)
                  enddo
                  call quadshear(xloc,yloc,0.1d0*sxy,flg(n))
                  call quadnorm(xloc,yloc,0.1d0*sxx,flg(n))
               endif
 390        continue
*     plot moho
            call pen(2)
            if (nmat.gt.1) then
               moho=nlay1
               call plot(real(x(moho)),real(y(moho)),3)
               do i= 2,nxx1
                  moho=moho+nyy1
                  call plot(real(x(moho)),real(y(moho)),2)
               enddo
               moho=nlay1
               call plot(-real(x(moho)),real(y(moho)),3)
               do i=2, nxx1
                  moho=moho+nyy1
                  call plot(-real(x(moho)),real(y(moho)),2)
               enddo
            endif
*     plot surface:
            moho=nyy1
            call plot(real(x(moho)),real(y(moho)),3)
            do i= 2,nxx1
               moho=moho+nyy1
               call plot(real(x(moho)),real(y(moho)),2)
            enddo
            moho=nyy1
            call plot(-real(x(moho)),real(y(moho)),3)
            do i=2, nxx1
               moho=moho+nyy1
               call plot(-real(x(moho)),real(y(moho)),2)
            enddo
         endif


* PLOT MESH AND SAVE MODEL (when it is the right time only!) -----------
*     initialize, scales
         if((iplot.ge.0 .or. negplot.eq.1) .and. (mod(iplot,mplot).eq.0
     &        .or.iplot.eq.nplot.or.iplot.eq.0)) then
            write (iwr,690) iplot
            iplotpage=iplotpage+1

            call page(iplotpage,iprob)
*     titles to appear at top of each plot
            write(8,8831) fstrain,grav,Fs,fname
*     Set up scaling for plotting in kilometers
            write(8,'(a)')'gsave'
            write(8,'(a)')'x y translate'
            write(8,'(a)')'xscale yscale scale' 
*     loop over elements to have position

* 5.3: initialisation for overburden calculation
            rhogez=maxobd

            do n=nod1, nem
*     calculate top node positions
*     need centroid, elxy, and w

* 5.3: new version of centroid returning grid column for n 
              call centroidint(n,icol,Sum,cenx,grav,2,nx,ny,nem,npe)
* 5.3: different initialisation because n begins at nod1               
               if (n.eq.nod1) icolold=icol

*      5.3:Depth summation of overburden with variable densities
*      5.3:To reset overburden to max overburden when new column is reached:
               if (icol.gt.icolold) then
                  icolold=icol
                  rhogez=maxobd
               endif

               rhogez=rhogez-celload(n-(icol-1)*ny)

               do i=1,npe
                  xloc(i)=elxy(i,1)
                  yloc(i)=elxy(i,2)
               enddo
c     Dieterich: strain-dependent rheology, brittle strength S

               S=1.d99
               do irb=1,nrb(n)

* 5.5: determines strain and/or rate weakening
        coeff=CoeffStr
	iwstr=iwstraint(n,irb)
	epsilv=str2t(n)
	epsill=epsil0n(n,irb)
	ws=weakSt(n,irb)
	wsr=weak(n,irb)
	sinv=e2(n)
	xpass=x(n)
	xmaxi=x(nem1)
	call weakening(coeff,vmcf,xpass,xmaxi,iwstr,epsilv,epsill,
     &   ws,wsr,sinv)

	CoeffStr=coeff


               S=min(S,
     &         (SC(n,irb) + SF(n,irb)*rhogez)*min(wmax(n,irb),CoeffStr))
               enddo
*     power law effective viscosity at strain rate invariant
               c1=dble(cmax)
               do ird=1,nrd(n)
                  c1 = min(c1,
     &                 cmu(n,ird)*(1./e2(n))**(1.-1./sn(n,ird)))
               enddo

               tau=2*c1*e2(n)*strnrate
*     Flag material near failure
               if (tau .gt. S) then
                  tau=-S
                  c1=tau/(2*e2(n)*strnrate)
               endif
               if (iprob.eq.2) c1=abs(c1)
*     plot the mesh,color as stress or strainrate
*     stress in element
             call stressorient(n,npe,elxy,w,c1,c2x,iel,sxx,sxy,flg(n))

                write (iwr,700) (elxy(i,1)/1000,
     &              elxy(i,2)/1000,i=1,npe),e2(n)
		
		if ((y(n).le.0.).and.(y(n+1).gt.0.)) then 
			
		write (ist,*) x(n-15),y(n-15),
     &           e2(n-15-n/(ny+1))/sqrt(2.),str2(n-15-n/(ny+1))/sqrt(2.)
		endif
               ni = nod(n,1)
               if(((n-1)/ny).lt.n25) then
                  if (iprob.eq.0) then
*     plot stress on right
                     call quadr(xloc,yloc,0.1d0*tau)
                  else
*     plot viscosity on right
* 5.3: if iprob=3, strain instead of viscosity or stress is on right
                     if (iprob.eq.3) then 
* 5.3: quads is quadv for strain
                        call quads(xloc,yloc,str2(n))
                     else
                        call quadv(xloc,yloc,c1)
                     endif
                  endif
*     plot strain rate on left
                  call quadl (xloc,yloc,e2(n))
               endif 
            enddo 
*     now plot velocity at local nodes 1 and 2
            if(nprnt .le.-1) then
               do n=nod1, nem
                  ni = nod(n,1)
                  if(mod(n-1,nx).lt.n25)then
                     call flowl(xp(ni),yp(ni),u(1,ni))
                     call flowr(xp(ni+1),yp(ni+1),u(1,ni+1))
                  endif
               enddo
            endif
*     plot moho
            call pen(2)
            if (nmat.gt.1) then
               moho=nlay1
               call plot(real(x(moho)),real(y(moho)),3)
               do i= 2,nxx1
                  moho=moho+nyy1
                  call plot(real(x(moho)),real(y(moho)),2)
               enddo
               moho=nlay1
               call plot(-real(x(moho)),real(y(moho)),3)
               do i=2, nxx1
                  moho=moho+nyy1
                  call plot(-real(x(moho)),real(y(moho)),2)
               enddo
            endif
*     plot surface:
            moho=nyy1
            call plot(real(x(moho)),real(y(moho)),3)
            do i= 2,nxx1
               moho=moho+nyy1
               call plot(real(x(moho)),real(y(moho)),2)
            enddo
            moho=nyy1
            call plot(-real(x(moho)),real(y(moho)),3)
            do i=2, nxx1
               moho=moho+nyy1
               call plot(-real(x(moho)),real(y(moho)),2)
            enddo
*     remember, all this was only if we wanted to save and plot the mesh...
         endif
 
* TAKE ONE STEP, (Still only if(integr=0)), eventually exit-------------
         iplot=iplot+1
         if (iplot.gt.nplot) goto 9000
         integr = 1

* DEFORM THE MESH (negative iplot for convergence of viscosity only,
* no deformations ------------------------------------------------------
         if(iplot.gt.0) then
            do i=1,nnm
               x(i)=x(i)+0.5d0*delt*gf(2*i-1)
               y(i)=y(i)+0.5d0*delt*gf(2*i)
            enddo
         endif

* IF INTEGR=1 WHEN ENTER THE POSTPROCESSOR ... -------------------------
      else
* replace mesh with positions saved in xp, whole time step.
         integr = 0
         if(iplot.gt.0) then
            do i=1,nnm
               x(i )=xp(i)+delt*gf(2*i-1)
               y(i)=yp(i)+delt*gf(2*i)
            enddo
         endif
* IN ANY CASE FOR INTEGR -----------------------------------------------
      endif
      
* LOOP if the result is not silly
      if (biggest.le.10000.) goto 101

* CLOSING STUFF (the end!!!) -------------------------------------------
 9000 continue



*     plotting is over
      call page(0,iprob)
      close (unit=8)


* EFECTIVE VISCOSITY ---------------------------------------------------

 
      print *, '   '
      print *,'postscript plot of lithosphere strain rate, stress: '
     $     ,psfile
 
      close(unit=iwr)
      close(unit=ist)

 400  format ('i=',i6,', z=',f12.2,', x=',f12.2,', Tm=',f12.2,
     & ', dT/dz=', f12.2)
 450  format(10x,'number of elements in the mesh........... =',i4,/,
     &     10x,'number of nodes in the mesh.............. =',i4,/,
     &     10x,'total number of equations in the model... =',i5,/)
 460  format(/,5x,'no. of specified degrees of freedom..... =',i3,/,
     *         5x,' location and boundary values are.......  ',
     *         /(5x,i5,5x,i5,1pe12.3))
 560  format(3x,'coordinates (x,y) of the global nodes:',/)
 580  format(-3p,6f12.6)
 590  format(/,5x,'half band width of global stiffness matrix =',i3,/)
 655  format(10x,'shear viscosity .................... =',1pe10.3,/,
     *       10x,'penalty parameter................... =',1pe10.3,/,
     *       10x,'cmin,cmax,U0=',e10.3,0p,4f10.3)
 660  format(10x,'element type................. =',i2,4i4/)
 666  format(f6.1,2f15.8,2f8.3)
 670  format('Strength law at reference strain rate ',a7,':',2g8.2)
 671  format('log10(v/v0) velocity weakening constant weak: ',f6.2)
 672  format('maximum weakening ratio wmax: ',f6.2)
 680  format('U0,tau0,L,H,nxx1,nyy1',/,2e12.4,2f12.4,2i6,/,
     &     'km/Ma,MPa,km,km,ND,ND')
 681  format (' iy    u_{bc}      v_{bc}        s_0        e_{xy}')

 690  format ('Grid saved at time step ',i8)
 700  format (3x,8f18.3,f18.6)
* postscript file formats-----------------------------------------------
 8800 format('x 18 add yt -30 add m ( A, n, Q, weak =   ',e9.3,f3.0,
     &     2e9.3,') show')
 8831 format('x -360 add yt -30 add m
     &     (strain,isostasy,Fs=',f6.3,f8.0,f5.1,' ',a15,') show')

      end
